<?php

function commerce_przelewy24_verify() {
  if (isset($_POST)) {
    if (isset($_POST['p24_id_sprzedawcy']) && isset($_POST['p24_session_id']) && isset($_POST['p24_order_id']) && isset($_POST['p24_order_id_full']) && isset($_POST['p24_kwota'])) {
      $p24_session_id = $_POST['p24_session_id'];
      $sa_post = explode('|', $p24_session_id);

      $p24_order_id = preg_replace('/\D/', '', $_POST['p24_order_id']);
      $p24_amount = preg_replace('/\D/', '', $_POST['p24_kwota']);
      $order_id = preg_replace('/\D/', '', $sa_post[0]);
      $order = commerce_order_load($order_id);

      // To get stored amount.
      $order_wrapper = entity_metadata_wrapper('commerce_order', $order);

      $fail_url = 'checkout/' . $order->order_id . '/payment/back/' . $order->data['payment_redirect_key'];

      if (isset($_POST['p24_error_code'])) {
        drupal_set_message(t('Przelewy24 payment has failed'), 'error');
        drupal_goto($fail_url);
      }

      $payment_method = commerce_payment_method_instance_load($order->data['payment_method']);

      if (isset($_POST['p24_crc']) && $payment_method['settings']['test_mode'] == 0) {
        $s_crc = md5($p24_session_id . '|' . $payment_method['settings']['seller_id']
         . '|' . $p24_amount . '|' . $payment_method['settings']['crc_key']);
        if ($s_crc != $_POST['p24_crc']) {
          drupal_set_message(t('Przelewy24 payment has failed'), 'error');
          drupal_goto(url($fail_url, array('absolute' => true)));
        }
      }

      $amount_stored = $order_wrapper->commerce_order_total->amount->value();
      if ($p24_amount == $amount_stored) {
        if (empty($transaction)) { //??
          $transaction = commerce_payment_transaction_new('przelewy24', $order->order_id);
          $transaction->created = time();
        }
        $transaction->instance_id = $payment_method['instance_id'];
        $transaction->remote_id = $order->order_id;
        $transaction->amount = $p24_amount;
        $transaction->currency_code = 'PL';
        $transaction->payload[REQUEST_TIME] = $_POST;

        if (commerce_przelewy24_payment_succeeded($p24_session_id, $p24_order_id, $payment_method['settings'], $amount_stored)) {
          $transaction->changed = time();

          $transaction->message = t('Przelewy24 payment has succeeded');
          $transaction->status = COMMERCE_PAYMENT_STATUS_SUCCESS;
          $transaction->remote_status = $transaction->status;
          commerce_checkout_complete($order);
          commerce_payment_transaction_save($transaction);
          drupal_goto(url('checkout/' . $order->order_id . '/payment/return/' . $order->data['payment_redirect_key'], array('absolute' => true)));
        }
        else {
          $transaction->changed = REQUEST_TIME;
          $transaction->status = COMMERCE_PAYMENT_STATUS_FAILURE;
          commerce_payment_transaction_save($transaction);
          drupal_goto($fail_url);
        }
      }
    }
  }
}

function commerce_przelewy24_payment_succeeded($p24_session_id, $p24_order_id, $payment_method_settings, $amount) {
  $p24_order_id = (int) $p24_order_id;

  $url_host = 'https://secure.przelewy24.pl';
  $url_query = array();

  if ($payment_method_settings['test_mode'] == 1) {
    $url_host = 'https://sandbox.przelewy24.pl';
  }
  else {
    $url_query[] = $payment_method_settings['crc_key'];
  }

  $url = $url_host . '/transakcja.php';
  $url_query[] = 'p24_id_sprzedawcy=' . $payment_method_settings['seller_id'];
  $url_query[] = 'p24_session_id=' . $p24_session_id;
  $url_query[] = 'p24_order_id=' . $p24_order_id;
  $url_query[] = 'p24_kwota=' . $amount;

  $user_agent = 'Mozilla/4.0 (compatible; MSIE 5.01; Windows NT 5.0)';

  $ch = curl_init();
  curl_setopt($ch, CURLOPT_POST, 1);
  if (count($url_query)) {
    curl_setopt($ch, CURLOPT_POSTFIELDS, join('&', $url_query));
  }

  curl_setopt($ch, CURLOPT_URL, $url);
  curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
  curl_setopt($ch, CURLOPT_USERAGENT, $user_agent);
  curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
  curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
  $result = curl_exec($ch);

  curl_close($ch);
  $T = explode(chr(13) . chr(10), $result);
  $res = FALSE;
  $RET = array();
  foreach ($T as $line) {
    $line = ereg_replace("[\n\r]", '', $line);
    if ($line != 'RESULT' && !$res) {
      continue;
    }
    if ($res) {
      $RET[] = $line;
    }
    else {
      $res = TRUE;
    }
  }

  if ($RET[0] == 'TRUE') {
    return TRUE;
  }

  return FALSE;
}
